class ErrorCode:
    ELEMENT_CLICK_INTERCEPTED = [64, 'element click intercepted']
    ELEMENT_IS_NOT_SELECTABLE = [15, 'element not selectable']
    ELEMENT_NOT_INTERACTABLE = [60, 'element not interactable']
    ELEMENT_NOT_VISIBLE = [11, 'element not visible']
    IME_ENGINE_ACTIVATION_FAILED = [31, 'ime engine activation failed']
    IME_NOT_AVAILABLE = [30, 'ime not available']
    INSECURE_CERTIFICATE = ['insecure certificate']
    INVALID_ARGUMENT = [61, 'invalid argument']
    INVALID_COOKIE_DOMAIN = [24, 'invalid cookie domain']
    INVALID_COORDINATES = ['invalid coordinates']
    INVALID_ELEMENT_COORDINATES = [29, 'invalid element coordinates']
    INVALID_ELEMENT_STATE = [12, 'invalid element state']
    INVALID_SELECTOR = [32, 'invalid selector']
    INVALID_SESSION_ID = ['invalid session id']
    INVALID_XPATH_SELECTOR = [51, 'invalid selector']
    INVALID_XPATH_SELECTOR_RETURN_TYPER = [52, 'invalid selector']
    JAVASCRIPT_ERROR = [17, 'javascript error']
    METHOD_NOT_ALLOWED = [405, 'unsupported operation']
    MOVE_TARGET_OUT_OF_BOUNDS = [34, 'move target out of bounds']
    NO_ALERT_OPEN = [27, 'no such alert']
    NO_SUCH_COOKIE = [62, 'no such cookie']
    NO_SUCH_ELEMENT = [7, 'no such element']
    NO_SUCH_FRAME = [8, 'no such frame']
    NO_SUCH_SHADOW_ROOT = ['no such shadow root']
    NO_SUCH_WINDOW = [23, 'no such window']
    SCRIPT_TIMEOUT = [28, 'script timeout']
    SESSION_NOT_CREATED = [33, 'session not created']
    STALE_ELEMENT_REFERENCE = [10, 'stale element reference']
    SUCCESS = 0
    TIMEOUT = [21, 'timeout']
    UNABLE_TO_CAPTURE_SCREEN = [63, 'unable to capture screen']
    UNABLE_TO_SET_COOKIE = [25, 'unable to set cookie']
    UNEXPECTED_ALERT_OPEN = [26, 'unexpected alert open']
    UNKNOWN_COMMAND = [9, 'unknown command']
    UNKNOWN_ERROR = [13, 'unknown error']
    UNKNOWN_METHOD = ['unknown method exception']
    XPATH_LOOKUP_ERROR = [19, 'invalid selector']