class mobile_command:
    ACTIVATE_APP= 'activateApp'
    ACTIVATE_IME_ENGINE= 'activateIMEEngine'
    BACKGROUND= 'background'
    CLEAR= 'clear'
    CLOSE_APP= 'closeApp'
    COMPARE_IMAGES= 'compareImages'
    CONTEXTS= 'getContexts'
    DEACTIVATE_IME_ENGINE= 'deactivateIMEEngine'
    END_TEST_COVERAGE= 'endTestCoverage'
    EXECUTE_DRIVER= 'executeDriver'
    FINGER_PRINT= 'fingerPrint'
    GET_ACTIVE_IME_ENGINE= 'getActiveEngine'
    GET_ALL_SESSIONS= 'getAllSessions'
    GET_APP_STRINGS= 'getAppStrings'
    GET_AVAILABLE_IME_ENGINES= 'getAvailableIMEEngines'
    GET_AVAILABLE_LOG_TYPES= 'getAvailableLogTypes'
    GET_CLIPBOARD= 'getClipboard'
    GET_CURRENT_ACTIVITY= 'getCurrentActivity'
    GET_CURRENT_CONTEXT= 'getCurrentContext'
    GET_CURRENT_PACKAGE= 'getCurrentPackage'
    GET_DEVICE_TIME_GET= 'getDeviceTimeGet'
    GET_DEVICE_TIME_POST= 'getDeviceTimePost'
    GET_DISPLAY_DENSITY= 'getDisplayDensity'
    GET_EVENTS= 'getLogEvents'
    GET_LOCATION= 'getLocation'
    GET_LOG= 'getLog'
    GET_NETWORK_CONNECTION= 'getNetworkConnection'
    GET_PERFORMANCE_DATA= 'getPerformanceData'
    GET_PERFORMANCE_DATA_TYPES= 'getPerformanceDataTypes'
    GET_SESSION= 'getSession'
    GET_SETTINGS= 'getSettings'
    GET_SYSTEM_BARS= 'getSystemBars'
    HIDE_KEYBOARD= 'hideKeyboard'
    INSTALL_APP= 'installApp'
    IS_APP_INSTALLED= 'isAppInstalled'
    IS_IME_ACTIVE= 'isIMEActive'
    IS_KEYBOARD_SHOWN= 'isKeyboardShown'
    IS_LOCKED= 'isLocked'
    KEY_EVENT= 'keyEvent'
    LAUNCH_APP= 'launchApp'
    LOCATION_IN_VIEW= 'locationInView'
    LOCK= 'lock'
    LOG_EVENT= 'logCustomEvent'
    LONG_PRESS_KEYCODE= 'longPressKeyCode'
    MAKE_GSM_CALL= 'makeGsmCall'
    MULTI_ACTION= 'multiAction'
    OPEN_NOTIFICATIONS= 'openNotifications'
    PRESS_KEYCODE= 'pressKeyCode'
    PULL_FILE= 'pullFile'
    PULL_FOLDER= 'pullFolder'
    PUSH_FILE= 'pushFile'
    QUERY_APP_STATE= 'queryAppState'
    REMOVE_APP= 'removeApp'
    REPLACE_KEYS= 'replaceKeys'
    RESET= 'reset'
    SEND_SMS= 'sendSms'
    SET_CLIPBOARD= 'setClipboard'
    SET_GSM_SIGNAL= 'setGsmSignal'
    SET_GSM_VOICE= 'setGsmVoice'
    SET_IMMEDIATE_VALUE= 'setImmediateValue'
    SET_LOCATION= 'setLocation'
    SET_NETWORK_CONNECTION= 'setNetworkConnection'
    SET_NETWORK_SPEED= 'setNetworkSpeed'
    SET_POWER_AC= 'setPowerAc'
    SET_POWER_CAPACITY= 'setPowerCapacity'
    SHAKE= 'shake'
    START_ACTIVITY= 'startActivity'
    START_RECORDING_SCREEN= 'startRecordingScreen'
    STOP_RECORDING_SCREEN= 'stopRecordingScreen'
    SWITCH_TO_CONTEXT= 'switchToContext'
    TERMINATE_APP= 'terminateApp'
    TOGGLE_LOCATION_SERVICES= 'toggleLocationServices'
    TOGGLE_TOUCH_ID_ENROLLMENT= 'toggleTouchIdEnrollment'
    TOGGLE_WIFI= 'toggleWiFi'
    TOUCH_ACTION= 'touchAction'
    TOUCH_ID= 'touchId'
    UNLOCK= 'unlock'
    UPDATE_SETTINGS= 'updateSettings'